@@ -1,10 +1,10 @@
-#define _POSIX_C_SOURCE 200809L
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
 #include <limits.h>
 #include <signal.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -14,7 +14,6 @@
 
 #include "config.h"
 #include "kanshi.h"
-#include "parser.h"
 #include "ipc.h"
 #include "wlr-output-management-unstable-v1-client-protocol.h"
 
@@ -323,6 +322,7 @@ static bool apply_profile(struct kanshi_state *state,
 			zwlr_output_configuration_head_v1_set_adaptive_sync(config_head,
 				profile_output->adaptive_sync);
 		}
+		zwlr_output_configuration_head_v1_destroy(config_head);
 	}
 
 	zwlr_output_configuration_v1_apply(config);
@@ -635,27 +635,6 @@ static struct kanshi_config *read_config(const char *config) {
 	return parse_config(config_path);
 }
 
-static void destroy_config(struct kanshi_config *config) {
-	struct kanshi_profile *profile, *tmp_profile;
-	wl_list_for_each_safe(profile, tmp_profile, &config->profiles, link) {
-		struct kanshi_profile_output *output, *tmp_output;
-		wl_list_for_each_safe(output, tmp_output, &profile->outputs, link) {
-			free(output->name);
-			wl_list_remove(&output->link);
-			free(output);
-		}
-		struct kanshi_profile_command *command, *tmp_command;
-		wl_list_for_each_safe(command, tmp_command, &profile->commands, link) {
-			free(command->command);
-			wl_list_remove(&command->link);
-			free(command);
-		}
-		wl_list_remove(&profile->link);
-		free(profile);
-	}
-	free(config);
-}
-
 bool kanshi_reload_config(struct kanshi_state *state,
 		kanshi_apply_done_func callback, void *data) {
 	fprintf(stderr, "reloading config\n");
@@ -728,11 +707,9 @@ int main(int argc, char *argv[]) {
 		.config = config,
 		.config_arg = config_arg,
 	};
-	int ret = EXIT_SUCCESS;
 #if KANSHI_HAS_VARLINK
 	if (kanshi_init_ipc(&state, listen_fd) != 0) {
-		ret = EXIT_FAILURE;
-		goto done;
+		return EXIT_FAILURE;
 	}
 #endif
 	wl_list_init(&state.heads);
@@ -747,16 +724,17 @@ int main(int argc, char *argv[]) {
 	if (state.output_manager == NULL) {
 		fprintf(stderr, "compositor doesn't support "
 			"wlr-output-management-unstable-v1\n");
-		ret = EXIT_FAILURE;
-		goto done;
+		return EXIT_FAILURE;
 	}
 
-	ret = kanshi_main_loop(&state);
+	int ret = kanshi_main_loop(&state);
 
-done:
 #if KANSHI_HAS_VARLINK
-	kanshi_free_ipc(&state);
+	kanshi_finish_ipc(&state);
 #endif
+	destroy_config(state.config);
+	zwlr_output_manager_v1_destroy(state.output_manager);
+	wl_registry_destroy(registry);
 	wl_display_disconnect(display);
 
 	return ret;
